﻿using System;
using System.Windows;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Input;
using WpfGoogleMapClient.Commands;

namespace WpfGoogleMapClient
{
	/// <summary>
	/// MainWindow.xaml の相互作用ロジック
	/// </summary>
	public partial class MainWindow : Window
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		public MainWindow()
		{
			string uri       = String.Format( "file://{0}res\\index.html", AppDomain.CurrentDomain.BaseDirectory );
			this.DataContext = new MainWindowViewModel( uri, OnMoveMap ) { Address = "新宿" };

			this.InitializeComponent();
		}

		/// <summary>
		/// 移動コマンドが実行された時に呼び出されます。
		/// </summary>
		/// <param name="address">移動先となる住所。</param>
		private void OnMoveMap( string address )
		{
			this._webBrowser.InvokeScript( "moveMap", address );
		}
	}

	/// <summary>
	/// MainWindow の Model と View を仲介するクラスです。
	/// </summary>
	class MainWindowViewModel
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="uri">ブラウザに表示するページの URI。</param>
		/// <param name="moveMap">移動コマンドが実行された時に呼び出されるコールバック メソッド。</param>
		public MainWindowViewModel( string uri, Action< string > moveMap )
		{
			this.Uri            = uri;
			this.MapHost        = new MapHost();
			this.MoveMapCommand = new DelegateCommand( () => { moveMap( this.Address ); } );
		}

		/// <summary>
		/// 現在設定されている Address の位置にマップを移動させるコマンドです。
		/// </summary>
		public ICommand MoveMapCommand { get; private set; }

		/// <summary>
		///  マップの操作を行う JavaScript に関連付けられるオブジェクトを取得します。
		/// </summary>
		public MapHost MapHost { get; private set; }

		/// <summary>
		/// マップに指定する住所を取得または設定します。
		/// </summary>
		public string Address { get; set; }

		/// <summary>
		/// ブラウザに表示するページの URI を取得します。
		/// </summary>
		public string Uri { get; private set; }
	}

	/// <summary>
	/// Google Map の操作を行う JavaScript に関連付けられるクラスです。
	/// </summary>
	[ComVisible( true )]
	public class MapHost : INotifyPropertyChanged
	{
		/// <summary>
		/// Google Map 上の緯度・経度を示す文字列を取得または設定します。
		/// </summary>
		public string Location
		{
			get
			{
				return this._location;
			}
			set
			{
				this._location = value;
				this.NotifyPropertyChanged( "Location" );
			}
		}

		/// <summary>
		/// Google Map 上の移動座標。
		/// </summary>
		private string _location;

		#region INotifyPropertyChanged メンバ

		/// <summary>
		/// プロパティが変更された事を通知するイベント ハンドラ。
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>
		/// プロパティの変更を通知します。
		/// </summary>
		/// <param name="names">変更されたプロパティ名のコレクション。</param>
		protected void NotifyPropertyChanged( params string[] names )
		{
			if( this.PropertyChanged != null )
			{
				foreach( string name in names )
				{
					this.PropertyChanged( this, new PropertyChangedEventArgs( name ) );
				}
			}
		}

		#endregion
	}
}
